#Performance Analysis by Andre Guerreiro#
#NS2 for 802.15.4#

#--------------------------------------
# Define Options
#--------------------------------------
set val(chan)		Channel/WirelessChannel		;#channel type
set val(prop)		Propagation/TwoRayGround	;#radio-propagation
set val(netif)		Phy/WirelessPhy/802_15_4
set val(mac)		Mac/802_15_4
set val(ifq)		Queue/DropTail/PriQueue		;#interface queue type
set val(ll)		LL				;#link layer type
set val(ant)		Antenna/OmniAntenna		;#antenna model
set val(ifqlen)		20				;#mac packet in ifq
set val(nn)		7				;#number of mobilenodes
set val(rp)		AODV				;#routng protocol
set val(x)		50		
set val(y)		50
set val(nam)		wpan2.nam
set val(traffic)	ftp				;#cbr/poisson/ftp
set val(errRate)	0    				;#error rate  %


#Set Frame Types

set Beacon 0
set Data 1
set Ack 2
set Mac 3
set AssociationRequest 1
set AssociationResponse 2
set DisassociationNotification 3
set DataRequest 4
set PANiDconflictNotification 5
set OrphanNotification 6
set BeaconRequest 7
set CoordinatorRealignment 8
set GTSrequest 9

#Set Scenarios for Temporal or Energetical Account
set ALL 0
set SBFL 1
set MBFL 2
set SYNCLOSS 3
set ORPH 4
set COORDREAL 5
set PANIDCONFLICT 6
set GTSREQUEST 7


# Read Command line arguments
proc getCmdArgu {argc argv} {
	global val
	for {set i 0} {$i < $argc} {incr i} {
		set arg [lindex $argv $i]
		if {[string range $arg 0 0] != "-"} continue
		set name [string range $arg 1 end]
		set val($name) [lindex $argv [expr $i+1]]
	}

}

getCmdArgu $argc $argv



set appTime1	30.0	;# in seconds
set appTime2	30.1	;# in seconds
set appTime3	30.2	;# in seconds
set appTime4	30.3	;# in seconds
set appTime5	30.4	;# in seconds
set appTime6	30.5	;# in seconds
set appTime7	50.6	;# in seconds
set appTime8	5.7	;# in seconds
set appTime9	15.8	;# in seconds
set appTime10	15.9	;# in seconds
set appTime11	16.0	;# in seconds
set appTime12	16.1	;# in seconds
set appTime13	16.2	;# in seconds
set stopTime	120	;# in seconds

set BO 4 ;
set SO 4 ;

#Initialize Global Variables
set ns_	[new Simulator]
set tracefd [open ./wpan2.tr w]
$ns_ trace-all $tracefd
#$ns_ use-newtrace

if { "$val(nam)" == "wpan2.nam"} {
      set namtrace [open ./$val(nam) w]
      $ns_ namtrace-all-wireless $namtrace $val(x) $val(y)
}

$ns_ puts-nam-traceall {# nam4wpan #} ;# inform nama that this is a trace file for wpan (special handling needed)

Mac/802_15_4 wpanCmd verbose on
Mac/802_15_4 wpanNam namStatus on ;#default = off (should be turned on before other 'wpanNam' commands can work)
#Mac/802_15_4 wpanNam ColFlashClr gold	;#default = gold


#Antenna/OmniAntenna set X_ 0
#Antenna/OmniAntenna set Y_ 0
#Antenna/OmniAntenna set Z_ 0.0864
#Antenna/OmniAntenna set Gt_ 1.0
#Antenna/OmniAntenna set Gr_ 1.0

#Phy/WirelessPhy set CPThresh_ 10

#The threshold values for the transmitter (CSThresh) and receiver (RXThresh). Values can be obtained from the "threshold" utility.
#Phy/WirelessPhy set Pt_ 0.001		;# Transmitter power = 0.0456
#Phy/WirelessPhy set freq_ 8.68e+08	;# frequency of operation = 8.68e+08
#Phy/WirelessPhy set L_ 1.0		;# Path loss = 1.0

#For model 'TwoRayGround'
set dist(5m)  7.69113e-06
set dist(9m)  2.37381e-06
set dist(10m) 1.92278e-06
set dist(11m) 1.58908e-06
set dist(12m) 1.33527e-06
set dist(13m) 1.13774e-06
set dist(14m) 9.81011e-07
set dist(15m) 8.54570e-07
set dist(16m) 7.51087e-07
set dist(20m) 4.80696e-07
set dist(25m) 3.07645e-07
set dist(30m) 2.13643e-07
set dist(35m) 1.56962e-07
set dist(40m) 1.20174e-07

Phy/WirelessPhy set CSThresh_ $dist(30m)
Phy/WirelessPhy set RXThresh_ $dist(30m) 

#set up topography object
set topo [new Topography]
$topo load_flatgrid $val(x) $val(y)

#Create God
set god_ [create-god $val(nn)]

set chan_1_ [new $val(chan)]

# configure node

$ns_ node-config -adhocRouting $val(rp) \
		-llType $val(ll) \
		-macType $val(mac) \
		-ifqType $val(ifq) \
		-ifqLen $val(ifqlen) \
		-antType $val(ant) \
		-propType $val(prop) \
		-phyType $val(netif) \
		-topoInstance $topo \
		-agentTrace OFF \
		-routerTrace OFF \
		-macTrace ON \
		-movementTrace OFF\
		-energyModel "EnergyModel" \
		-initialEnergy 13000 \
		-idlePower 0.0 \
                -rxPower 0.03 \
                -txPower 0.03 \
#		-rxPower 0.0648 \
#		-txPower 0.0744\
		-channel $chan_1_

# insert error model
#if {$val(errRate) != 0} {
#	$ns_ node-config -errProc UniformErr
#}

proc UniformErr {} {
        set err [new ErrorModel]
        $err unit pkt 
$err set period_ 0.02
        $err set rate_ 0.0017
#[expr 5.0 / 100.0]
        $err drop-target [new Agent/Null]
        return $err
}
#$ns_ node-config -IncomingErrProc UniformErr
proc UniformEmm {} {
#$ error_model 
#        set emm [new ErrorModel/Expo [list 100 10] pkt]
 set emm [new ErrorModel]
$emm ranvar [new RandomVariable/Exponential]
#$emm 
	$emm set period_ 0.02
        $emm unit pkt 
        $emm set rate_ [expr 5.0 / 100.0]
        $emm drop-target [new Agent/Null]
        return $emm
}


#0.000608
#$ns_ node-config -IncomingErrProc UniformEmm
#$ns_ node-config -IncomingErrProc UniformErr -OutgoingErrProc UniformErr

#packet_t PT_CBR = 2;
#packet_t PT_ACK = 5;
#packet_t PT_MAC = 35

#	proc UniformEmm {} {

#error model
#set em [new SelectErrorModel]

#$em set enable_ 1 #true if this error module is turned on
#$em set markecn_ true # mark ecn instead of dropping on corruption?
#$em set unit_ packet;        # error unit in pkts, bytes, or time
# int firstTime_;         // to not corrupt first packet in byte model
#$em set rate_ 1.0 	#uniform error rate in pkt or byte
#$em set bandwidth_ 2Mb  #bandwidth of the link
#$em set pkt_type_ 2
#$em set drop_cycle_ 10
#$em set drop_offset_ 1
#$em set debug_ true
#$em set delay_pkt_ false #delay packet instead of dropping
#$em set delay_ 0 #time to delay packet
#$eM ranvar [new RandomVariable/Uniform] #the underlying random variate generator
#return $em
#	}








#	proc UniformErr {} {
#		set err [new ErrorModel]
#$err set rate_ 1.0
#		$err unit packet
#$err ranvar [new RandomVariable/Uniform]
#$err drop-target [new Agent/Null]
#		return $err
#	}

#set node_(0) [$ns_ node]
#	$node_(0) random-motion 0 ;
for {set i 0} {$i <4} {incr i} {
	set node_($i) [$ns_ node]
	$node_($i) random-motion 0 ;#disable random motion
}

#$ns_ node-config -IncomingErrProc UniformEmm 
for {set i 4} {$i < 7} {incr i} {
	set node_($i) [$ns_ node]
	$node_($i) random-motion 0 ;#disable random motion
}
#$ns_ node-config -errProc UniformEmm


source ./wpan2.scn


$ns_ at 0.0	"$node_(0) NodeLabel PAN Coor"
$ns_ at 0.0     "$node_(0) NodeClr blue"
#auxiliar time variables
set j 0.5
set g 65.0
set avg 1.0
#start nodes and GTS on/off
for {set i 1} {$i < $val(nn)} {incr i} {
#	$ns_ at 0.0	"$node_($i) NodeLabel RFD"

$ns_ at $j	"$node_($i) sscs startDevice 0 0 0 $BO $SO"
#if { $i < 8} {
#$ns_ at $g	"$node_($i) sscs txOption GTS On"
#set g [expr $g + $avg]
#}
set j [expr $j + $avg]
}
#$ns_ at 30.0	"$node_(1) sscs randomFaultInjection $stopTime 0 8"
#$ns_ at 30.0	"$node_(2) sscs randomFaultInjection $stopTime 0 8"
$ns_ at 42.0	"$node_(1) sscs txOption GTS On"
$ns_ at 43.0	"$node_(2) sscs txOption GTS On"
$ns_ at 44.0	"$node_(3) sscs txOption GTS On"
$ns_ at 43.0	"$node_(4) sscs txOption GTS On"
$ns_ at 41.0	"$node_(5) sscs txOption GTS On"
$ns_ at 48.0	"$node_(6) sscs txOption GTS On"

$ns_ at 0.0	"$node_(0) sscs startPANCoord 1 $BO $SO"		;#startPANCoord <txBeacon=1> <BO=3> <SO=3>	
#$ns_ at 0.5	"$node_(1) sscs startDevice 0 0 0 $BO $SO"	;#startDevice <isFFD=1> <assoPermit=1> <txBeacon=0> <BO=3> <SO=3> , see p802_15_4sscs.cc

Mac/802_15_4 wpanNam PlaybackRate 3ms ;#set Nam Playback Rate

$ns_ at $appTime1 "puts \"\n Transmitting data ... \n\""

#Setup traffic flow between nodes

proc cbrtraffic { src dst interval starttime} {
	global ns_ node_
	set udp_($src) [new Agent/UDP]
	eval $ns_ attach-agent \$node_($src) \$udp_($src)
	set null_($dst) [new Agent/Null]
	eval $ns_ attach-agent \$node_($dst) \$null_($dst)
	set cbr_($src) [new Application/Traffic/CBR]
	eval \$cbr_($src) set packetSize_ 100
	eval \$cbr_($src) set interval_ $interval
	eval \$cbr_($src) set random_ 0
#	eval \$cbr_($src) set rate_ 100
	#eval \$cbr_($src) set maxpkts_ 10000
	eval \$cbr_($src) attach-agent \$udp_($src)
	eval $ns_ connect \$udp_($src) \$null_($dst)
	$ns_ at $starttime "$cbr_($src) start"
}

proc poissontraffic { src dst interval starttime} {
	global ns_ node_
	set udp($src) [new Agent/UDP]
	eval $ns_ attach-agent \$node_($src) \$udp($src)
	set null($dst) [new Agent/Null]
	eval $ns_ attach-agent \$node_($dst) \$null($dst)
	set expl($src) [new Application/Traffic/Exponential]
	eval \$expl($src) set packetSize_ 100 ;#Bytes 
	eval \$expl($src) set burst_time_ 0
	eval \$expl($src) set idle_time_ [expr $interval*1000.0-100.0*8/250]ms ;# idle_time + pkt_tx_time = interval
	eval \$expl($src) set rate_ 250k
	eval \$expl($src) attach-agent \$udp($src)
	eval $ns_ connect \$udp($src) \$null($dst)
	$ns_ at $starttime "$expl($src) start"
}

if { ("$val(traffic)" == "cbr") || ("$val(traffic)" == "poisson") } {
	puts "\nTraffic: $val(traffic)"
	Mac/802_15_4 wpanCmd ack4data off
	puts [format "Acknowledgement for data: %s" [Mac/802_15_4 wpanCmd ack4data]]
	$ns_ at $appTime1 "Mac/802_15_4 wpanNam PlaybackRate 0.5ms"
	$ns_ at [expr $appTime1 + 0.5] "Mac/802_15_4 wpanNam PlaybackRate 1.5ms"
	#cbrtraffic 1 0 0.5 $appTime1
	#cbrtraffic 2 0 0.5 $appTime2
	#cbrtraffic 3 0 0.5 $appTime3
	#cbrtraffic 4 0 0.5 $appTime4
	#cbrtraffic 5 0 0.5 $appTime5
	#cbrtraffic 6 0 0.5 $appTime6
#	$val(traffic)traffic 7 0 0.1 $appTime7
	#$val(traffic)traffic 8 0 0.001 $appTime8
	#$val(traffic)traffic 9 0 0.001 $appTime9
	#$val(traffic)traffic 10 0 0.001 $appTime10
	#$val(traffic)traffic 11 0 0.001 $appTime11
	#$val(traffic)traffic 12 0 0.001 $appTime12
	#$val(traffic)traffic 13 0 0.001 $appTime13
	$ns_ at $appTime1 "$ns_ trace-annotate \"(at $appTime1) cbrtraffic traffic from node 0 to node 1\""
	$ns_ at $appTime2 "$ns_ trace-annotate \"(at $appTime2) cbrtraffic traffic from node 0 to node 2\""
	$ns_ at $appTime3 "$ns_ trace-annotate \"(at $appTime3) cbrtraffic traffic from node 0 to node 3\""
	$ns_ at $appTime4 "$ns_ trace-annotate \"(at $appTime4) poissontraffic traffic from node 0 to node 4\""
	$ns_ at $appTime5 "$ns_ trace-annotate \"(at $appTime5) poissontraffic traffic from node 0 to node 5\""
	$ns_ at $appTime6 "$ns_ trace-annotate \"(at $appTime6) poissontraffic traffic from node 0 to node 6\""
#	$ns_ at $appTime7 "$ns_ trace-annotate \"(at $appTime1) $val(traffic) traffic from node 6 to node 0\""
	#$ns_ at $appTime8 "$ns_ trace-annotate \"(at $appTime2) $val(traffic) traffic from node 8 to node 0\""
	#$ns_ at $appTime9 "$ns_ trace-annotate \"(at $appTime3) $val(traffic) traffic from node 9 to node 0\""
	#$ns_ at $appTime10 "$ns_ trace-annotate \"(at $appTime4) $val(traffic) traffic from node 10 to node 0\""
	#$ns_ at $appTime11 "$ns_ trace-annotate \"(at $appTime5) $val(traffic) traffic from node 11 to node 0\""
	#$ns_ at $appTime12 "$ns_ trace-annotate \"(at $appTime6) $val(traffic) traffic from node 12 to node 0\""
	#$ns_ at $appTime13 "$ns_ trace-annotate \"(at $appTime1) $val(traffic) traffic from node 13 to node 0\""

	Mac/802_15_4 wpanNam FlowClr -p AODV -c tomato
	Mac/802_15_4 wpanNam FlowClr -p ARP -c green
	Mac/802_15_4 wpanNam FlowClr -p MAC -s 0 -d -1 -c navy
	if { "$val(traffic)" == "cbr" } {
		set pktType cbr
	} else {
	    set pktType exp
	}
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 1 -d 0 -c blue
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 2 -d 0 -c green4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 3 -d 0 -c cyan4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 4 -d 0 -c blue
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 5 -d 0 -c green4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 6 -d 0 -c cyan4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 7 -d 0 -c blue
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 8 -d 0 -c green4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 9 -d 0 -c cyan4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 10 -d 0 -c blue
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 11 -d 0 -c green4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 12 -d 0 -c cyan4
	Mac/802_15_4 wpanNam FlowClr -p $pktType -s 13 -d 0 -c cyan4

}

proc ftptraffic { src dst starttime } {
	global ns_ node_
	set tcp($src) [new Agent/TCP]
	eval \$tcp($src) set packetSize_ 50
	set sink($dst) [new Agent/TCPSink]
	eval $ns_ attach-agent \$node_($src) \$tcp($src)
	eval $ns_ attach-agent \$node_($dst) \$sink($dst)
	eval $ns_ connect \$tcp($src) \$sink($dst)
	set ftp($src) [new Application/FTP]
	eval \$ftp($src) attach-agent \$tcp($src)
	$ns_ at $starttime "$ftp($src) start"
}

if { "$val(traffic)" == "ftp" } {
	puts "\nTraffic: ftp"
	#Mac/802_15_4 wpanCmd ack4data off
	puts [format "Acknowledgement for data: %s" [Mac/802_15_4 wpanCmd ack4data]]
	$ns_ at $appTime1 "Mac/802_15_4 wpanNam PlaybackRate 0.20ms"
	$ns_ at [expr $appTime1 + 0.5] "Mac/802_15_4 wpanNam PlaybackRate 1.5ms"
	ftptraffic 1 0 $appTime1
	ftptraffic 2 0 $appTime2
	ftptraffic 3 0 $appTime3
	ftptraffic 4 0 $appTime4
	ftptraffic 5 0 $appTime5
	ftptraffic 6 0 $appTime6
	$ns_ at $appTime1 "$ns_ trace-annotate \"(at $appTime1) ftp traffic from node 1 to node 0\" "
	$ns_ at $appTime2 "$ns_ trace-annotate \"(at $appTime2) ftp traffic from node 2 to node 0\" "
	$ns_ at $appTime3 "$ns_ trace-annotate \"(at $appTime3) ftp traffic from node 3 to node 0\" "
	$ns_ at $appTime4 "$ns_ trace-annotate \"(at $appTime4) ftp traffic from node 4 to node 0\" "
	$ns_ at $appTime5 "$ns_ trace-annotate \"(at $appTime5) ftp traffic from node 5 to node 0\" "
	$ns_ at $appTime6 "$ns_ trace-annotate \"(at $appTime6) ftp traffic from node 6 to node 0\" "
	Mac/802_15_4 wpanNam FlowClr -p AODV -c tomato
	Mac/802_15_4 wpanNam FlowClr -p ARP -c green
	Mac/802_15_4 wpanNam FlowClr -p MAC -s 0 -d -1 -c navy
	Mac/802_15_4 wpanNam FlowClr -p tcp -s 1 -d 0 -c blue
	Mac/802_15_4 wpanNam FlowClr -p ack -s 0 -d 1 -c blue
	Mac/802_15_4 wpanNam FlowClr -p tcp -s 2 -d 0 -c green4
	Mac/802_15_4 wpanNam FlowClr -p ack -s 0 -d 2 -c green4
	Mac/802_15_4 wpanNam FlowClr -p tcp -s 3 -d 0 -c cyan4
	Mac/802_15_4 wpanNam FlowClr -p ack -s 0 -d 3 -c cyan4
	Mac/802_15_4 wpanNam FlowClr -p tcp -s 4 -d 0 -c blue
	Mac/802_15_4 wpanNam FlowClr -p ack -s 0 -d 4 -c blue
	Mac/802_15_4 wpanNam FlowClr -p tcp -s 5 -d 0 -c green4
	Mac/802_15_4 wpanNam FlowClr -p ack -s 0 -d 5 -c green4
	Mac/802_15_4 wpanNam FlowClr -p tcp -s 6 -d 0 -c cyan4
	Mac/802_15_4 wpanNam FlowClr -p ack -s 0 -d 6 -c cyan4
}

#defines the node size in nam
for {set i 0} {$i < $val(nn)} {incr i} {
	$ns_ initial_node_pos $node_($i) 2
	#above command defines the node initial position in nam and the node size
}

#Tell nodes when the simulation ends
for {set i 0} {$i < $val(nn)} {incr i} {
	$ns_ at $stopTime "$node_($i) reset";
}

#comandos da camada SSCS WPAN
# --- $node sscs startPANCoord	<txBeacon = 1> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startDevice <isFFD = 1> <assoPermit = 1> <txBeacon = 0> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startCTPANCoord <txBeacon = 1> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startCTDevice <isFFD = 1> <assoPermit = 1> <txBeacon = 0> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startBeacon <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs stopBeacon
#--Added Functions----------------------------------
# --- $node sscs txOption GTS <On/Off>
# --- $node sscs panConflict <0 = off/ 1= on>

#comandos do Fault Injector e Temporal/Energy account
# --- $node sscs enableEnergyAccount <scenario=0> <start= 0.0> <end=0.0>
# --- $node sscs enableTemporalAccount <scenario=0>
# --- $node sscs faultInjection <frame=0> <ntimes= 0> <node=0>
# --- $node sscs randomFaultInjection <stop=simulationTime> <frame=0> <ntimes= 0> <node=0>

# frame:
#{0="M_BEACON",1="M_CM1_AssoReq",2="M_CM2_AssoRsp",3="M_CM3_DisaNtf",4="M_CM4_DataReq",5="M_CM5_PidCNtf",6="M_CM6_OrphNtf",7="M_CM7_Bcn-Req",8="M_CM8_CoorRal",9="M_CM9_GTS-Req",10="M_ACK",11="M_UNKN"}
#Inacessibility events
#$ns_ at 65.0	"$node_(0) sscs enableTemporalAccount $ALL";
$ns_ at 40.0	"$node_(1) sscs enableTemporalAccount $ALL";
#$ns_ at 40.0	"$node_(1) sscs enableEnergyAccount 43.305792 47.237952";
$ns_ at 40.0	"$node_(1) sscs enableEnergyAccount 0 0 0";
$ns_ at 40.0	"$node_(1) sscs faultInjection $Beacon 5";
$ns_ at 41.2	"$node_(1) sscs faultInjection $CoordinatorRealignment 5";
#$ns_ at 0.0	"$node_(1) sscs panConflict 1"
#$ns_ at 0.0	"$node_(0) sscs panConflict 1"
#$ns_ at 60.0	"$node_(1) sscs faultInjection 6 1"
#$ns_ at 40\.0	"$node_(2) sscs faultInjection 0 8"
#$ns_ at 180.0	"$node_(1) sscs disableFaultInjection"
#$ns_ at 62.0	"$node_(1) sscs disableFaultInjection"
#$ns_ at 43.0 "puts \"NS faultInjection...\n\""



$ns_ at $stopTime "stop"
$ns_ at $stopTime "puts \"NS EXITING...\n\""
$ns_ at $stopTime "$ns_ halt"

proc stop {} {
	global ns_ tracefd appTime1 val env
	$ns_ flush-trace
	close $tracefd
	set hasDISPLAY 0
	foreach index [array names env] {
		#puts "$index: $env($index)"
		if { ("$index" == "DISPLAY") && ("$env($index)" != "")} {
 			set hasDISPLAY 1		
		}
	}	
	if { ("$val(nam)" == "wpan2.nam") && ("$hasDISPLAY" == "1") } {
		exec nam wpan2.nam &
	}
}

puts "\n Starting Simulation..."

$ns_ run
